@@ -1,6 +1,6 @@
 ;;; company-dabbrev-code.el --- dabbrev-like company-mode backend for code  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2009, 2011, 2014  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2011, 2013-2016, 2021  Free Software Foundation, Inc.
 
 ;; Author: Nikolaj Schumacher
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 
 ;;; Commentary:
@@ -76,7 +76,7 @@ also `company-dabbrev-code-time-limit'."
           "\\(\\sw\\|\\s_\\)*\\_>"))
 
 ;;;###autoload
-(defun company-dabbrev-code (command &optional arg &rest ignored)
+(defun company-dabbrev-code (command &optional arg &rest _ignored)
   "dabbrev-like `company-mode' backend for code.
 The backend looks for all symbols in the current buffer that aren't in
 comments or strings."
@@ -97,6 +97,7 @@ comments or strings."
                      (`code company-dabbrev-code-modes)
                      (`all `all))
                    (not company-dabbrev-code-everywhere))))
+    (kind 'text)
     (ignore-case company-dabbrev-code-ignore-case)
     (duplicates t)))
 
