@@ -1,6 +1,6 @@
 ;;; company-keywords.el --- A company backend for programming language keywords
 
-;; Copyright (C) 2009-2011, 2016  Free Software Foundation, Inc.
+;; Copyright (C) 2009-2011, 2013-2018, 2020-2022  Free Software Foundation, Inc.
 
 ;; Author: Nikolaj Schumacher
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 
 ;;; Commentary:
@@ -27,6 +27,15 @@
 
 (require 'company)
 (require 'cl-lib)
+(eval-when-compile (require 'make-mode))
+
+(defgroup company-keywords nil
+  "Completion backend for keywords."
+  :group 'company)
+
+(defcustom company-keywords-ignore-case nil
+  "Non-nil to ignore case in completion candidates."
+  :type 'boolean)
 
 (defun company-keywords-upper-lower (&rest lst)
   ;; Upcase order is different for _.
@@ -52,13 +61,23 @@
      "xor" "xor_eq")
     (c-mode
      ;; from https://en.cppreference.com/w/c/keyword
-     "_Alignas" "_Alignof" "_Atomic" "_Bool" "_Complex" "_Generic" "_Imaginary"
-     "_Noreturn" "_Static_assert" "_Thread_local"
+     "_Alignas" "_Alignof" "_Atomic" "_Bool" "_Complex"
+     "_Decimal128" "_Decimal32" "_Decimal64" "_Generic" "_Imaginary"
+     "_Noreturn" "_Static_assert" "_Thread_local" "__asm__" "asm"
      "auto" "break" "case" "char" "const" "continue" "default" "do"
      "double" "else" "enum" "extern" "float" "for" "goto" "if" "inline"
      "int" "long" "register" "restrict" "return" "short" "signed" "sizeof"
      "static" "struct" "switch" "typedef" "union" "unsigned" "void" "volatile"
      "while")
+    (crystal-mode
+     ;; from https://github.com/crystal-lang/crystal-book/issues/124#issuecomment-1008311227
+    "abstract" "alias" "annotation" "as" "as?" "asm" "begin" "break" "case" "class"
+    "def" "do" "else" "elsif" "end" "ensure" "enum" "extend" "false" "for" "fun"
+    "if" "in" "include" "instance_sizeof" "is_a?" "lib" "macro" "module" "next"
+    "nil" "nil?" "of" "offsetof" "out" "pointerof" "private" "protected" "require"
+    "rescue" "responds_to?" "return" "select" "self" "sizeof" "struct" "super"
+    "then" "true" "type" "typeof" "uninitialized" "union" "unless" "until" "verbatim"
+    "when" "while" "with" "yield")
     (csharp-mode
      "abstract" "add" "alias" "as" "base" "bool" "break" "byte" "case"
      "catch" "char" "checked" "class" "const" "continue" "decimal" "default"
@@ -86,6 +105,29 @@
      "super" "switch" "synchronized" "template" "this" "throw" "true" "try"
      "typedef" "typeid" "typeof" "ubyte" "ucent" "uint" "ulong" "union"
      "unittest" "ushort" "version" "void" "volatile" "wchar" "while" "with")
+    (elixir-mode
+     ;; from https://hexdocs.pm/elixir/Kernel.html
+     "__CALLER__" "__DIR__" "__ENV__" "__MODULE__" "__STACKTRACE__"
+     "__aliases__" "__block__" "abs" "alias" "alias!" "and" "apply"
+     "binary_part" "binary_slice" "binding" "bit_size" "byte_size" "case" "ceil"
+     "cond" "dbg" "def" "defdelegate" "defexception" "defguard" "defguardp"
+     "defimpl" "defmacro" "defmacrop" "defmodule" "defoverridable" "defp"
+     "defprotocol" "defstruct" "destructure" "div" "elem" "exit" "floor" "fn"
+     "for" "function_exported?" "get_and_update_in" "get_in" "hd" "if" "import"
+     "in" "inspect" "is_atom" "is_binary" "is_bitstring" "is_boolean"
+     "is_exception" "is_float" "is_function" "is_integer" "is_list" "is_map"
+     "is_map_key" "is_nil" "is_number" "is_pid" "is_port" "is_reference"
+     "is_struct" "is_tuple" "length" "macro_exported?" "make_ref" "map_size"
+     "match?" "max" "min" "node" "not" "or" "pop_in" "put_elem" "put_in" "quote"
+     "raise" "receive" "rem" "require" "reraise" "round" "self" "send" "spawn"
+     "spawn_link" "spawn_monitor" "struct" "struct!" "super" "tap" "then"
+     "throw" "tl" "to_charlist" "to_string" "trunc" "try" "tuple_size" "unless"
+     "unquote" "unquote_splicing" "update_in" "use" "var!" "with")
+    (erlang-mode
+     ;; from https://www.erlang.org/docs/20/reference_manual/introduction.html#id63536
+     "after" "and" "andalso" "band" "begin" "bnot" "bor" "bsl" "bsr" "bxor"
+     "case" "catch" "cond" "div" "end" "fun" "if" "let" "not" "of" "or" "orelse"
+     "receive" "rem" "try" "when" "xor")
     (f90-mode .
      ;; from f90.el
      ;; ".AND." ".GE." ".GT." ".LT." ".LE." ".NE." ".OR." ".TRUE." ".FALSE."
@@ -180,6 +222,18 @@
      "internal" "is" "lateinit" "nested" "null" "object" "open" "out" "override"
      "package" "private" "protected" "public" "return" "super" "this" "throw"
      "trait" "true" "try" "typealias" "val" "var" "when" "while")
+    (lua-mode
+     ;; https://www.lua.org/manual/5.3/manual.html
+     "and" "break" "do" "else" "elseif" "end" "false" "for" "function" "goto" "if"
+     "in" "local" "nil" "not" "or" "repeat" "return" "then" "true" "until" "while")
+    (nim-mode
+     ;; https://nim-lang.org/docs/manual.html#lexical-analysis-identifiers-amp-keywords
+     "addr" "and" "as" "asm" "bind" "block" "break" "case" "cast" "concept" "const" "continue"
+     "converter" "defer" "discard" "distinct" "div" "do" "elif" "else" "end" "enum" "except"
+     "export" "finally" "for" "from" "func" "if" "import" "in" "include" "interface" "is" "isnot"
+     "iterator" "let" "macro" "method" "mixin" "mod" "nil" "not" "notin" "object" "of" "or" "out"
+     "proc" "ptr" "raise" "ref" "return" "shl" "shr" "static" "template" "try" "tuple" "type"
+     "using" "var" "when" "while" "xor" "yield")
     (objc-mode
      "@catch" "@class" "@encode" "@end" "@finally" "@implementation"
      "@interface" "@private" "@protected" "@protocol" "@public"
@@ -218,17 +272,24 @@
      "ucfirst" "umask" "undef" "unless" "unlink" "unpack" "unshift" "untie"
      "until" "use" "utime" "values" "vec" "wait" "waitpid"
      "wantarray" "warn" "while" "write" "x" "xor" "y")
-    (php-mode
+    (php-mode ;; https://www.php.net/manual/reserved.php
+     "Closure" "Error" "Exception" "Generator" "Throwable"
      "__CLASS__" "__DIR__" "__FILE__" "__FUNCTION__" "__LINE__" "__METHOD__"
-     "__NAMESPACE__" "_once" "abstract" "and" "array" "as" "break" "case"
-     "catch" "cfunction" "class" "clone" "const" "continue" "declare"
-     "default" "die" "do" "echo" "else" "elseif" "empty" "enddeclare"
-     "endfor" "endforeach" "endif" "endswitch" "endwhile" "eval" "exception"
-     "exit" "extends" "final" "for" "foreach" "function" "global"
-     "goto" "if" "implements" "include" "instanceof" "interface"
-     "isset" "list" "namespace" "new" "old_function" "or" "php_user_filter"
-     "print" "private" "protected" "public" "require" "require_once" "return"
-     "static" "switch" "this" "throw" "try" "unset" "use" "var" "while" "xor")
+     "__NAMESPACE__" "__TRAIT__"
+     "abstract" "and" "array" "as" "bool" "break" "callable" "case" "catch"
+     "class" "clone" "const" "continue" "declare" "default" "die" "do" "echo"
+     "else" "elseif" "empty" "enddeclare" "endfor" "endforeach" "endif"
+     "endswitch" "endwhile" "enum" "eval" "exit" "extends" "false" "final" "finally"
+     "float" "fn" "for" "foreach" "function" "global" "goto" "if"
+     "implements" "include" "include_once" "instanceof" "insteadof" "interface"
+     "isset" "iterable" "list" "match" "namespace" "new" "null" "object" "or"
+     "print" "private" "protected" "public" "readonly" "require" "require_once"
+     "return" "self" "static" "string" "switch" "this" "throw" "trait" "true"
+     "try" "unset" "use" "var" "void" "while" "xor" "yield" "yield from")
+    (purescript-mode ;; purescript-font-lock.el
+     "ado" "case" "class" "data" "default" "deriving" "do" "else" "if" "import"
+     "in" "infix" "infixl" "infixr" "instance" "let" "module" "newtype" "of"
+     "then" "type" "where")
     (python-mode
      ;; https://docs.python.org/3/reference/lexical_analysis.html#keywords
      "False" "None" "True" "and" "as" "assert" "break" "class" "continue" "def"
@@ -248,6 +309,68 @@
      "false" "fn" "for" "if" "impl" "in" "let" "loop" "macro" "match" "mod"
      "move" "mut" "pub" "ref" "return" "self" "static" "struct" "super"
      "trait" "true" "type" "unsafe" "use" "where" "while")
+    ; Extract from R7RS-small Tex: https://small.r7rs.org/
+    (scheme-mode
+     "abs" "acos" "angle" "append" "apply" "asin" "assoc" "assq" "assv"
+     "atan" "binary-port?" "body" "boolean=?" "boolean?" "bytevector"
+     "bytevector-append" "bytevector-copy" "bytevector-copy!"
+     "bytevector-length" "bytevector-u8-ref" "bytevector-u8-set!"
+     "bytevector?" "caaaar" "caaadr" "caaar" "caadar" "caaddr" "caadr"
+     "caar" "cadaar" "cadadr" "cadar" "caddar" "cadddr" "caddr" "cadr"
+     "call-with-port" "call-with-values" "car" "car-internal" "cdaaar"
+     "cdaadr" "cdaar" "cdadar" "cdaddr" "cdadr" "cdar" "cddaar" "cddadr"
+     "cddar" "cdddar" "cddddr" "cdddr" "cddr" "cdr" "ceiling"
+     "char->integer" "char-alphabetic?" "char-ci<=?" "char-ci<?"
+     "char-ci=?" "char-ci>=?" "char-ci>?" "char-downcase" "char-foldcase"
+     "char-lower-case?" "char-numeric?" "char-ready?" "char-upcase"
+     "char-upper-case?" "char-whitespace?" "char<=?" "char<?" "char=?"
+     "char>=?" "char>?" "char?" "close-input-port" "close-output-port"
+     "close-port" "command-line" "complex?" "cons" "cos"
+     "current-error-port" "current-input-port" "current-jiffy"
+     "current-output-port" "current-second" "delete-file" "denominator"
+     "digit-value" "display" "dynamic-wind" "emergency-exit" "environment"
+     "eof-object" "eof-object?" "eq?" "equal?" "eqv?" "error"
+     "error-object-irritants" "error-object-message" "error-object?" "eval"
+     "even?" "exact" "exact-integer-sqrt" "exact-integer?" "exact?" "exit"
+     "exp" "expt" "features" "file-error?" "file-exists?" "finite?" "floor"
+     "floor-quotient" "floor-remainder" "floor/" "flush-output-port" "gcd"
+     "get-environment-variable" "get-environment-variables"
+     "get-output-bytevector" "get-output-string" "imag-part" "inexact"
+     "inexact?" "infinite?" "input-port-open?" "input-port?"
+     "integer->char" "integer?" "interaction-environment"
+     "jiffies-per-second" "lcm" "length" "list" "list->string"
+     "list->vector" "list-copy" "list-ref" "list-set!" "list-tail" "list?"
+     "load" "log" "magnitude" "make-bytevector" "make-list"
+     "make-parameter" "make-polar" "make-promise" "make-rectangular"
+     "make-string" "make-vector" "max" "member" "memq" "memv" "min"
+     "modulo" "nan?" "negative?" "newline" "nil" "not" "null-environment"
+     "null?" "number->string" "number?" "numerator" "odd?"
+     "open-binary-input-file" "open-binary-output-file"
+     "open-input-bytevector" "open-input-file" "open-input-string"
+     "open-output-bytevector" "open-output-file" "open-output-string"
+     "output-port-open?" "output-port?" "pair?" "peek-char" "peek-u8"
+     "port?" "positive?" "procedure?" "promise?" "quasiquote" "quote"
+     "quotient" "raise" "raise-continuable" "rational?" "rationalize"
+     "read" "read-bytevector" "read-bytevector!" "read-char" "read-error?"
+     "read-line" "read-string" "read-u8" "real-part" "real?" "remainder"
+     "reverse" "round" "scheme-report-environment" "set!" "set-car!"
+     "set-cdr!" "setcar" "sin" "sqrt" "square" "string" "string->list"
+     "string->number" "string->symbol" "string->utf" "string->vector"
+     "string-append" "string-ci<=?" "string-ci<?" "string-ci=?"
+     "string-ci>=?" "string-ci>?" "string-copy" "string-copy!"
+     "string-downcase" "string-fill!" "string-foldcase" "string-for-each"
+     "string-length" "string-map" "string-ref" "string-set!"
+     "string-upcase" "string<=?" "string<?" "string=?" "string>=?"
+     "string>?" "string?" "substring" "symbol->string" "symbol=?" "symbol?"
+     "tan" "textual-port?" "truncate" "truncate-quotient"
+     "truncate-remainder" "truncate/" "u8-ready?" "unquote"
+     "unquote-splicing" "utf->string" "values" "vector" "vector->list"
+     "vector->string" "vector-append" "vector-copy" "vector-copy!"
+     "vector-fill!" "vector-for-each" "vector-length" "vector-map"
+     "vector-ref" "vector-set!" "vector?" "with-exception-handler"
+     "with-input-from-file" "with-output-to-file" "write"
+     "write-bytevector" "write-char" "write-shared" "write-simple"
+     "write-string" "write-u8" "zero?")
     (scala-mode
      "abstract" "case" "catch" "class" "def" "do" "else" "extends" "false"
      "final" "finally" "for" "forSome" "if" "implicit" "import" "lazy" "match"
@@ -290,9 +413,31 @@
     (cperl-mode . perl-mode)
     (jde-mode . java-mode)
     (ess-julia-mode . julia-mode)
+    (phps-mode . php-mode)
     (enh-ruby-mode . ruby-mode))
   "Alist mapping major-modes to sorted keywords for `company-keywords'.")
 
+(with-eval-after-load 'make-mode
+  (mapc
+   (lambda (mode-stmnts)
+     (setf (alist-get (car mode-stmnts) company-keywords-alist)
+           (cl-remove-duplicates
+            (sort (append makefile-special-targets-list
+                          (cl-mapcan #'identity
+                                     (mapcar
+                                      #'split-string
+                                      (cl-remove-if-not
+                                       #'stringp
+                                       (symbol-value (cdr mode-stmnts))))))
+                  #'string<)
+            :test #'string=)))
+   '((makefile-automake-mode . makefile-automake-statements)
+     (makefile-gmake-mode    . makefile-gmake-statements)
+     (makefile-makepp-mode   . makefile-makepp-statements)
+     (makefile-bsdmake-mode  . makefile-bsdmake-statements)
+     (makefile-imake-mode    . makefile-statements)
+     (makefile-mode          . makefile-statements))))
+
 ;;;###autoload
 (defun company-keywords (command &optional arg &rest ignored)
   "`company-mode' backend for programming language keywords."
@@ -303,12 +448,14 @@
                  (not (company-in-string-or-comment))
                  (or (company-grab-symbol) 'stop)))
     (candidates
-     (let ((completion-ignore-case nil)
+     (let ((completion-ignore-case company-keywords-ignore-case)
            (symbols (cdr (assq major-mode company-keywords-alist))))
        (all-completions arg (if (consp symbols)
                                 symbols
                               (cdr (assq symbols company-keywords-alist))))))
-    (sorted t)))
+    (kind 'keyword)
+    (sorted t)
+    (ignore-case company-keywords-ignore-case)))
 
 (provide 'company-keywords)
 ;;; company-keywords.el ends here
