@@ -1,6 +1,6 @@
 ;;; company-tests.el --- company-mode test helpers  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2011, 2013-2016  Free Software Foundation, Inc.
+;; Copyright (C) 2011, 2013-2016, 2022  Free Software Foundation, Inc.
 
 ;; Author: Dmitry Gutov
 
@@ -17,9 +17,10 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 (require 'company)
+(require 'ert)
 
 (defvar company-dir (file-name-directory (or load-file-name
                                              buffer-file-name)))
@@ -36,4 +37,9 @@
     (let ((this-command command))
       (run-hooks 'post-command-hook))))
 
+(defalias 'company--equal-including-properties
+  (if (version< "29" emacs-version)
+      'equal-including-properties
+    'ert-equal-including-properties))
+
 (provide 'company-tests)
