@@ -1,6 +1,6 @@
 ;;; clang-tests.el --- company-mode tests  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015  Free Software Foundation, Inc.
+;; Copyright (C) 2015, 2017, 2023  Free Software Foundation, Inc.
 
 ;; Author: Dmitry Gutov
 
@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 (require 'company-tests)
 (require 'company-clang)
@@ -49,3 +49,16 @@
         (s (propertize "s" 'meta "struct (anonymous) s")))
     (should (null (company-clang 'annotation u)))
     (should (null (company-clang 'annotation s)))))
+
+(ert-deftest company-clang-pattern-to-match-empty ()
+  (should (equal (company-clang--pattern-to-match "else")
+                 "else")))
+
+(ert-deftest company-clang-pattern-to-match-with-paren ()
+  (should (equal (company-clang--pattern-to-match "else if (<#expression#>)")
+                 "else if ")))
+
+(ert-deftest company-clang-pattern-to-match-no-paren ()
+  (should (equal (company-clang--pattern-to-match
+                  "[#size_t#]sizeof(<#expression-or-type#>)")
+                 "sizeof")))
